@@ -16,7 +16,7 @@ public class Constants {
     /**
      * The build date is updated for each public release.
      */
-    public static final String BUILD_DATE = "2015-01-16";
+    public static final String BUILD_DATE = "2016-01-21";
 
     /**
      * The build date of the last stable release.
@@ -26,7 +26,7 @@ public class Constants {
     /**
      * The build id is incremented for each public release.
      */
-    public static final int BUILD_ID = 185;
+    public static final int BUILD_ID = 191;
 
     /**
      * The build id of the last stable release.
@@ -217,7 +217,7 @@ public class Constants {
      * The default maximum length of an LOB that is stored with the record
      * itself, and not in a separate place.
      */
-    public static final int DEFAULT_MAX_LENGTH_INPLACE_LOB = 128;
+    public static final int DEFAULT_MAX_LENGTH_INPLACE_LOB = 256;
 
     /**
      * The default value for the maximum transaction log size.
@@ -505,6 +505,11 @@ public class Constants {
      */
     public static final int VIEW_INDEX_CACHE_SIZE = 64;
 
+    /**
+     * The maximum number of entries in query statistics.
+     */
+    public static final int QUERY_STATISTICS_MAX_ENTRIES = 100;
+
     private Constants() {
         // utility class
     }
