@@ -9,16 +9,19 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
+import java.util.LinkedList;
+import java.util.Map;
 import java.util.Random;
-
 import org.h2.api.ErrorCode;
 import org.h2.command.Command;
 import org.h2.command.CommandInterface;
 import org.h2.command.Parser;
 import org.h2.command.Prepared;
+import org.h2.command.dml.Query;
 import org.h2.command.dml.SetTypes;
 import org.h2.constraint.Constraint;
 import org.h2.index.Index;
+import org.h2.index.ViewIndex;
 import org.h2.jdbc.JdbcConnection;
 import org.h2.message.DbException;
 import org.h2.message.Trace;
@@ -28,11 +31,14 @@ import org.h2.mvstore.db.TransactionStore.Change;
 import org.h2.mvstore.db.TransactionStore.Transaction;
 import org.h2.result.LocalResult;
 import org.h2.result.Row;
+import org.h2.result.SortOrder;
 import org.h2.schema.Schema;
 import org.h2.store.DataHandler;
 import org.h2.store.InDoubtTransaction;
 import org.h2.store.LobStorageFrontend;
+import org.h2.table.SubQueryInfo;
 import org.h2.table.Table;
+import org.h2.table.TableFilter;
 import org.h2.util.New;
 import org.h2.util.SmallLRUCache;
 import org.h2.value.Value;
@@ -85,7 +91,7 @@ public class Session extends SessionWithState {
     private String currentSchemaName;
     private String[] schemaSearchPath;
     private Trace trace;
-    private HashMap<String, Value> unlinkLobMap;
+    private HashMap<String, Value> removeLobMap;
     private int systemIdentifier;
     private HashMap<String, Procedure> procedures;
     private boolean undoLogEnabled = true;
@@ -108,6 +114,27 @@ public class Session extends SessionWithState {
     private final int queryCacheSize;
     private SmallLRUCache<String, Command> queryCache;
     private long modificationMetaID = -1;
+    private SubQueryInfo subQueryInfo;
+    private int parsingView;
+    private int preparingQueryExpression;
+    private volatile SmallLRUCache<Object, ViewIndex> viewIndexCache;
+    private HashMap<Object, ViewIndex> subQueryIndexCache;
+    private boolean joinBatchEnabled;
+    private boolean forceJoinOrder;
+
+    /**
+     * Temporary LOBs from result sets. Those are kept for some time. The
+     * problem is that transactions are committed before the result is returned,
+     * and in some cases the next transaction is already started before the
+     * result is read (for example when using the server mode, when accessing
+     * metadata methods). We can't simply free those values up when starting the
+     * next transaction, because they would be removed too early.
+     */
+    private LinkedList<TimeoutValue> temporaryResultLobs;
+
+    /**
+     * The temporary LOBs that need to be removed on commit.
+     */
     private ArrayList<Value> temporaryLobs;
 
     private Transaction transaction;
@@ -127,6 +154,93 @@ public class Session extends SessionWithState {
         this.currentSchemaName = Constants.SCHEMA_MAIN;
     }
 
+    public void setForceJoinOrder(boolean forceJoinOrder) {
+        this.forceJoinOrder = forceJoinOrder;
+    }
+
+    public boolean isForceJoinOrder() {
+        return forceJoinOrder;
+    }
+
+    public void setJoinBatchEnabled(boolean joinBatchEnabled) {
+        this.joinBatchEnabled = joinBatchEnabled;
+    }
+
+    public boolean isJoinBatchEnabled() {
+        return joinBatchEnabled;
+    }
+
+    /**
+     * Create a new row for a table.
+     *
+     * @param data the values
+     * @param memory whether the row is in memory
+     * @return the created row
+     */
+    public Row createRow(Value[] data, int memory) {
+        return database.createRow(data, memory);
+    }
+
+    /**
+     * Add a subquery info on top of the subquery info stack.
+     *
+     * @param masks the mask
+     * @param filters the filters
+     * @param filter the filter index
+     * @param sortOrder the sort order
+     */
+    public void pushSubQueryInfo(int[] masks, TableFilter[] filters, int filter,
+            SortOrder sortOrder) {
+        subQueryInfo = new SubQueryInfo(subQueryInfo, masks, filters, filter, sortOrder);
+    }
+
+    /**
+     * Remove the current subquery info from the stack.
+     */
+    public void popSubQueryInfo() {
+        subQueryInfo = subQueryInfo.getUpper();
+    }
+
+    public SubQueryInfo getSubQueryInfo() {
+        return subQueryInfo;
+    }
+
+    public void setParsingView(boolean parsingView) {
+        // It can be recursive, thus implemented as counter.
+        this.parsingView += parsingView ? 1 : -1;
+        assert this.parsingView >= 0;
+    }
+
+    public boolean isParsingView() {
+        assert parsingView >= 0;
+        return parsingView != 0;
+    }
+
+    /**
+     * Optimize a query. This will remember the subquery info, clear it, prepare
+     * the query, and reset the subquery info.
+     *
+     * @param query the query to prepare
+     */
+    public void optimizeQueryExpression(Query query) {
+        // we have to hide current subQueryInfo if we are going to optimize
+        // query expression
+        SubQueryInfo tmp = subQueryInfo;
+        subQueryInfo = null;
+        preparingQueryExpression++;
+        try {
+            query.prepare();
+        } finally {
+            subQueryInfo = tmp;
+            preparingQueryExpression--;
+        }
+    }
+
+    public boolean isPreparingQueryExpression() {
+        assert preparingQueryExpression >= 0;
+        return preparingQueryExpression != 0;
+    }
+
     @Override
     public ArrayList<String> getClusterServers() {
         return new ArrayList<String>();
@@ -158,14 +272,13 @@ public class Session extends SessionWithState {
             old = variables.remove(name);
         } else {
             // link LOB values, to make sure we have our own object
-            value = value.link(database,
+            value = value.copy(database,
                     LobStorageFrontend.TABLE_ID_SESSION_VARIABLE);
             old = variables.put(name, value);
         }
         if (old != null) {
-            // close the old value (in case it is a lob)
-            old.unlink(database);
-            old.close();
+            // remove the old value (in case it is a lob)
+            old.remove();
         }
     }
 
@@ -443,7 +556,13 @@ public class Session extends SessionWithState {
             }
         }
         Parser parser = new Parser(this);
-        command = parser.prepareCommand(sql);
+        try {
+            command = parser.prepareCommand(sql);
+        } finally {
+            // we can't reuse sub-query indexes, so just drop the whole cache
+            subQueryIndexCache = null;
+        }
+        command.prepareJoinBatch();
         if (queryCache != null) {
             if (command.isCacheable()) {
                 queryCache.put(sql, command);
@@ -497,14 +616,7 @@ public class Session extends SessionWithState {
             // (create/drop table and so on)
             database.commit(this);
         }
-        if (temporaryLobs != null) {
-            for (Value v : temporaryLobs) {
-                if (!v.isLinked()) {
-                    v.close();
-                }
-            }
-            temporaryLobs.clear();
-        }
+        removeTemporaryLobs(true);
         if (undoLog.size() > 0) {
             // commit the rows when using MVCC
             if (database.isMultiVersion()) {
@@ -536,6 +648,31 @@ public class Session extends SessionWithState {
         endTransaction();
     }
 
+    private void removeTemporaryLobs(boolean onTimeout) {
+        if (temporaryLobs != null) {
+            for (Value v : temporaryLobs) {
+                if (!v.isLinkedToTable()) {
+                    v.remove();
+                }
+            }
+            temporaryLobs.clear();
+        }
+        if (temporaryResultLobs != null && temporaryResultLobs.size() > 0) {
+            long keepYoungerThan = System.currentTimeMillis() -
+                    database.getSettings().lobTimeout;
+            while (temporaryResultLobs.size() > 0) {
+                TimeoutValue tv = temporaryResultLobs.getFirst();
+                if (onTimeout && tv.created >= keepYoungerThan) {
+                    break;
+                }
+                Value v = temporaryResultLobs.removeFirst().value;
+                if (!v.isLinkedToTable()) {
+                    v.remove();
+                }
+            }
+        }
+    }
+
     private void checkCommitRollback() {
         if (commitOrRollbackDisabled && locks.size() > 0) {
             throw DbException.get(ErrorCode.COMMIT_ROLLBACK_NOT_ALLOWED);
@@ -543,15 +680,16 @@ public class Session extends SessionWithState {
     }
 
     private void endTransaction() {
-        if (unlinkLobMap != null && unlinkLobMap.size() > 0) {
-            // need to flush the transaction log, because we can't unlink lobs
-            // if the commit record is not written
-            database.flush();
-            for (Value v : unlinkLobMap.values()) {
-                v.unlink(database);
-                v.close();
+        if (removeLobMap != null && removeLobMap.size() > 0) {
+            if (database.getMvStore() == null) {
+                // need to flush the transaction log, because we can't unlink
+                // lobs if the commit record is not written
+                database.flush();
             }
-            unlinkLobMap = null;
+            for (Value v : removeLobMap.values()) {
+                v.remove();
+            }
+            removeLobMap = null;
         }
         unlockAll();
     }
@@ -617,7 +755,7 @@ public class Session extends SessionWithState {
                         row = t.getRow(this, key);
                     } else {
                         op = UndoLogRecord.DELETE;
-                        row = new Row(value.getList(), Row.MEMORY_CALCULATE);
+                        row = createRow(value.getList(), Row.MEMORY_CALCULATE);
                     }
                     row.setKey(key);
                     UndoLogRecord log = new UndoLogRecord(t, op, row);
@@ -671,6 +809,7 @@ public class Session extends SessionWithState {
         if (!closed) {
             try {
                 database.checkPowerOff();
+                removeTemporaryLobs(false);
                 cleanTempTables(true);
                 undoLog.clear();
                 database.removeSession(this);
@@ -819,11 +958,11 @@ public class Session extends SessionWithState {
         if (trace != null && !closed) {
             return trace;
         }
-        String traceModuleName = Trace.JDBC + "[" + id + "]";
+        String traceModuleName = "jdbc[" + id + "]";
         if (closed) {
             return new TraceSystem(null).getTrace(traceModuleName);
         }
-        trace = database.getTrace(traceModuleName);
+        trace = database.getTraceSystem().getTrace(traceModuleName);
         return trace;
     }
 
@@ -1092,29 +1231,28 @@ public class Session extends SessionWithState {
     }
 
     /**
-     * Remember that the given LOB value must be un-linked (disconnected from
-     * the table) at commit.
+     * Remember that the given LOB value must be removed at commit.
      *
      * @param v the value
      */
-    public void unlinkAtCommit(Value v) {
-        if (SysProperties.CHECK && !v.isLinked()) {
+    public void removeAtCommit(Value v) {
+        if (SysProperties.CHECK && !v.isLinkedToTable()) {
             DbException.throwInternalError();
         }
-        if (unlinkLobMap == null) {
-            unlinkLobMap = New.hashMap();
+        if (removeLobMap == null) {
+            removeLobMap = New.hashMap();
+            removeLobMap.put(v.toString(), v);
         }
-        unlinkLobMap.put(v.toString(), v);
     }
 
     /**
-     * Do not unlink this LOB value at commit any longer.
+     * Do not remove this LOB value at commit any longer.
      *
      * @param v the value
      */
-    public void unlinkAtCommitStop(Value v) {
-        if (unlinkLobMap != null) {
-            unlinkLobMap.remove(v.toString());
+    public void removeAtCommitStop(Value v) {
+        if (removeLobMap != null) {
+            removeLobMap.remove(v.toString());
         }
     }
 
@@ -1263,6 +1401,31 @@ public class Session extends SessionWithState {
         }
     }
 
+    /**
+     * Get the view cache for this session. There are two caches: the subquery
+     * cache (which is only use for a single query, has no bounds, and is
+     * cleared after use), and the cache for regular views.
+     *
+     * @param subQuery true to get the subquery cache
+     * @return the view cache
+     */
+    public Map<Object, ViewIndex> getViewIndexCache(boolean subQuery) {
+        if (subQuery) {
+            // for sub-queries we don't need to use LRU because the cache should
+            // not grow too large for a single query (we drop the whole cache in
+            // the end of prepareLocal)
+            if (subQueryIndexCache == null) {
+                subQueryIndexCache = New.hashMap();
+            }
+            return subQueryIndexCache;
+        }
+        SmallLRUCache<Object, ViewIndex> cache = viewIndexCache;
+        if (cache == null) {
+            viewIndexCache = cache = SmallLRUCache.newInstance(Constants.VIEW_INDEX_CACHE_SIZE);
+        }
+        return cache;
+    }
+
     /**
      * Remember the result set and close it as soon as the transaction is
      * committed (if it needs to be closed). This is done to delete temporary
@@ -1443,12 +1606,30 @@ public class Session extends SessionWithState {
         closeTemporaryResults();
     }
 
+    /**
+     * Clear the view cache for this session.
+     */
+    public void clearViewIndexCache() {
+        viewIndexCache = null;
+    }
+
     @Override
     public void addTemporaryLob(Value v) {
-        if (temporaryLobs == null) {
-            temporaryLobs = new ArrayList<Value>();
+        if (v.getType() != Value.CLOB && v.getType() != Value.BLOB) {
+            return;
+        }
+        if (v.getTableId() == LobStorageFrontend.TABLE_RESULT ||
+                v.getTableId() == LobStorageFrontend.TABLE_TEMP) {
+            if (temporaryResultLobs == null) {
+                temporaryResultLobs = new LinkedList<TimeoutValue>();
+            }
+            temporaryResultLobs.add(new TimeoutValue(v));
+        } else {
+            if (temporaryLobs == null) {
+                temporaryLobs = new ArrayList<Value>();
+            }
+            temporaryLobs.add(v);
         }
-        temporaryLobs.add(v);
     }
 
     /**
@@ -1468,4 +1649,24 @@ public class Session extends SessionWithState {
         long transactionSavepoint;
     }
 
+    /**
+     * An object with a timeout.
+     */
+    public static class TimeoutValue {
+
+        /**
+         * The time when this object was created.
+         */
+        final long created = System.currentTimeMillis();
+
+        /**
+         * The value.
+         */
+        final Value value;
+
+        TimeoutValue(Value v) {
+            this.value = v;
+        }
+
+    }
 }
