@@ -12,6 +12,10 @@ import java.nio.ByteBuffer;
  */
 public class WriteBuffer {
 
+    /**
+     * The maximum size of the buffer in order to be re-used after a clear
+     * operation.
+     */
     private static final int MAX_REUSE_CAPACITY = 4 * 1024 * 1024;
 
     /**
@@ -19,9 +23,24 @@ public class WriteBuffer {
      */
     private static final int MIN_GROW = 1024 * 1024;
 
-    private ByteBuffer reuse = ByteBuffer.allocate(MIN_GROW);
+    /**
+     * The buffer that is used after a clear operation.
+     */
+    private ByteBuffer reuse;
 
-    private ByteBuffer buff = reuse;
+    /**
+     * The current buffer (may be replaced if it is too small).
+     */
+    private ByteBuffer buff;
+
+    public WriteBuffer(int initialSize) {
+        reuse = ByteBuffer.allocate(initialSize);
+        buff = reuse;
+    }
+
+    public WriteBuffer() {
+        this(MIN_GROW);
+    }
 
     /**
      * Write a variable size integer.
@@ -297,13 +316,18 @@ public class WriteBuffer {
         return buff;
     }
 
-    private void grow(int len) {
+    private void grow(int additional) {
         ByteBuffer temp = buff;
-        int needed = len - temp.remaining();
-        int grow = Math.max(needed, MIN_GROW);
+        int needed = additional - temp.remaining();
+        // grow at least MIN_GROW
+        long grow = Math.max(needed, MIN_GROW);
         // grow at least 50% of the current size
         grow = Math.max(temp.capacity() / 2, grow);
-        int newCapacity = temp.capacity() + grow;
+        // the new capacity is at most Integer.MAX_VALUE
+        int newCapacity = (int) Math.min(Integer.MAX_VALUE, temp.capacity() + grow);
+        if (newCapacity < needed) {
+            throw new OutOfMemoryError("Capacity: " + newCapacity + " needed: " + needed);
+        }
         try {
             buff = ByteBuffer.allocate(newCapacity);
         } catch (OutOfMemoryError e) {
