@@ -13,6 +13,7 @@ import java.io.InputStream;
 import java.io.Reader;
 import java.sql.PreparedStatement;
 import java.sql.SQLException;
+
 import org.h2.engine.Constants;
 import org.h2.engine.SysProperties;
 import org.h2.message.DbException;
@@ -102,6 +103,9 @@ public class ValueLobDb extends Value implements Value.ValueClob,
                 if (len == 0) {
                     break;
                 }
+                byte[] data = new String(buff, 0, len).getBytes("UTF-8");
+                out.write(data);
+                tmpPrecision += len;
             }
         } finally {
             out.close();
@@ -202,9 +206,9 @@ public class ValueLobDb extends Value implements Value.ValueClob,
     }
 
     @Override
-    public boolean isLinked() {
-        return tableId != LobStorageFrontend.TABLE_ID_SESSION_VARIABLE &&
-                small == null;
+    public boolean isLinkedToTable() {
+        return small == null &&
+                tableId >= 0;
     }
 
     public boolean isStored() {
@@ -212,7 +216,7 @@ public class ValueLobDb extends Value implements Value.ValueClob,
     }
 
     @Override
-    public void close() {
+    public void remove() {
         if (fileName != null) {
             if (tempFile != null) {
                 tempFile.stopAutoDelete();
@@ -229,24 +233,10 @@ public class ValueLobDb extends Value implements Value.ValueClob,
     }
 
     @Override
-    public void unlink(DataHandler database) {
-        if (small == null &&
-                tableId != LobStorageFrontend.TABLE_ID_SESSION_VARIABLE) {
-            database.getLobStorage().setTable(this,
-                    LobStorageFrontend.TABLE_ID_SESSION_VARIABLE);
-            tableId = LobStorageFrontend.TABLE_ID_SESSION_VARIABLE;
-        }
-    }
-
-    @Override
-    public Value link(DataHandler database, int tabId) {
+    public Value copy(DataHandler database, int tableId) {
         if (small == null) {
-            if (tableId == LobStorageFrontend.TABLE_TEMP) {
-                database.getLobStorage().setTable(this, tabId);
-                this.tableId = tabId;
-            } else {
-                return handler.getLobStorage().copyLob(this, tabId, getPrecision());
-            }
+            Value v2 = handler.getLobStorage().copyLob(this, tableId, getPrecision());
+            return v2;
         } else if (small.length > database.getMaxLengthInplaceLob()) {
             LobStorageInterface s = database.getLobStorage();
             Value v;
@@ -255,7 +245,9 @@ public class ValueLobDb extends Value implements Value.ValueClob,
             } else {
                 v = s.createClob(getReader(), getPrecision());
             }
-            return v.link(database, tabId);
+            Value v2 = v.copy(database, tableId);
+            v.remove();
+            return v2;
         }
         return this;
     }
