@@ -18,6 +18,8 @@ import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.lang.reflect.Modifier;
 import java.lang.reflect.Proxy;
+import java.nio.channels.FileChannel;
+import java.nio.channels.FileLock;
 import java.sql.Connection;
 import java.sql.DriverManager;
 import java.sql.ResultSet;
@@ -32,8 +34,7 @@ import java.util.LinkedList;
 import java.util.SimpleTimeZone;
 import org.h2.jdbc.JdbcConnection;
 import org.h2.message.DbException;
-import org.h2.message.TraceSystem;
-import org.h2.store.FileLock;
+import org.h2.store.fs.FilePath;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.utils.ProxyCodeGenerator;
 import org.h2.test.utils.ResultVerifier;
@@ -49,16 +50,16 @@ public abstract class TestBase {
      */
     public static final String BASE_TEST_DIR = "./data";
 
-    /**
-     * The temporary directory.
-     */
-    protected static final String TEMP_DIR = "./data/temp";
-
     /**
      * An id used to create unique file names.
      */
     protected static int uniqueId;
 
+    /**
+     * The temporary directory.
+     */
+    private static final String TEMP_DIR = "./data/temp";
+
     /**
      * The base directory to write test databases.
      */
@@ -115,6 +116,7 @@ public abstract class TestBase {
      */
     public TestBase init(TestAll conf) throws Exception {
         baseDir = getTestDir("");
+        FileUtils.createDirectories(baseDir);
         System.setProperty("java.io.tmpdir", TEMP_DIR);
         this.config = conf;
         return this;
@@ -154,15 +156,6 @@ public abstract class TestBase {
             if (e instanceof OutOfMemoryError) {
                 throw (OutOfMemoryError) e;
             }
-        } finally {
-            try {
-                FileUtils.deleteRecursive("memFS:", false);
-                FileUtils.deleteRecursive("nioMemFS:", false);
-                FileUtils.deleteRecursive("memLZF:", false);
-                FileUtils.deleteRecursive("nioMemLZF:", false);
-            } catch (RuntimeException e) {
-                e.printStackTrace();
-            }
         }
     }
 
@@ -260,7 +253,6 @@ public abstract class TestBase {
             return name;
         }
         if (admin) {
-            ; // TODO testing
             // name = addOption(name, "RETENTION_TIME", "10");
             // name = addOption(name, "WRITE_DELAY", "10");
         }
@@ -307,7 +299,7 @@ public abstract class TestBase {
         } else if (config.throttle > 0) {
             url = addOption(url, "THROTTLE", "" + config.throttle);
         }
-        url = addOption(url, "LOCK_TIMEOUT", "50");
+        url = addOption(url, "LOCK_TIMEOUT", "" + config.lockTimeout);
         if (config.diskUndo && admin) {
             url = addOption(url, "MAX_MEMORY_UNDO", "3");
         }
@@ -331,9 +323,6 @@ public abstract class TestBase {
         if (config.defrag) {
             url = addOption(url, "DEFRAG_ALWAYS", "TRUE");
         }
-        if (config.nestedJoins) {
-            url = addOption(url, "NESTED_JOINS", "TRUE");
-        }
         return "jdbc:h2:" + url;
     }
 
@@ -456,6 +445,21 @@ public abstract class TestBase {
      */
     protected void fail(String string) {
         lastPrint = 0;
+        if (string.length() > 100) {
+            // avoid long strings with special characters, because they are slow
+            // to display in Eclipse
+            char[] data = string.toCharArray();
+            for (int i = 0; i < data.length; i++) {
+                char c = data[i];
+                if (c >= 128 || c < 32) {
+                    data[i] = (char) ('a' + (c & 15));
+                    string = null;
+                }
+            }
+            if (string == null) {
+                string = new String(data);
+            }
+        }
         println(string);
         throw new AssertionError(string);
     }
@@ -474,18 +478,31 @@ public abstract class TestBase {
         System.err.println("ERROR: " + s + " " + e.toString()
                 + " ------------------------------");
         e.printStackTrace();
-        try {
-            TraceSystem ts = new TraceSystem(null);
-            FileLock lock = new FileLock(ts, "error.lock", 1000);
-            lock.lock(FileLock.LOCK_FILE);
-            FileWriter fw = new FileWriter("error.txt", true);
-            PrintWriter pw = new PrintWriter(fw);
-            e.printStackTrace(pw);
-            pw.close();
-            fw.close();
-            lock.unlock();
-        } catch (Throwable t) {
-            t.printStackTrace();
+        // synchronize on this class, because file locks are only visible to
+        // other JVMs
+        synchronized (TestBase.class) {
+            try {
+                // lock
+                FileChannel fc = FilePath.get("error.lock").open("rw");
+                FileLock lock;
+                while (true) {
+                    lock = fc.tryLock();
+                    if (lock != null) {
+                        break;
+                    }
+                    Thread.sleep(10);
+                }
+                // append
+                FileWriter fw = new FileWriter("error.txt", true);
+                PrintWriter pw = new PrintWriter(fw);
+                e.printStackTrace(pw);
+                pw.close();
+                fw.close();
+                // unlock
+                lock.release();
+            } catch (Throwable t) {
+                t.printStackTrace();
+            }
         }
         System.err.flush();
     }
@@ -510,7 +527,7 @@ public abstract class TestBase {
      * @param millis the time in milliseconds
      * @param s the message
      */
-    static void printlnWithTime(long millis, String s) {
+    static synchronized void printlnWithTime(long millis, String s) {
         SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");
         s = dateFormat.format(new java.util.Date()) + " " +
                 formatTime(millis) + " " + s;
@@ -739,6 +756,9 @@ public abstract class TestBase {
                 String s = expected.substring(0, i);
                 if (!actual.startsWith(s)) {
                     expected = expected.substring(0, i) + "<*>" + expected.substring(i);
+                    if (al > 20) {
+                        expected = "@" + i + " " + expected;
+                    }
                     break;
                 }
             }
@@ -1000,17 +1020,17 @@ public abstract class TestBase {
     /**
      * Check that executing the specified query results in the specified error.
      *
-     * @param expectedErrorMessage the expected error message
+     * @param expectedErrorCode the expected error code
      * @param stat the statement
      * @param sql the SQL statement to execute
      */
-    protected void assertThrows(String expectedErrorMessage, Statement stat,
+    protected void assertThrows(int expectedErrorCode, Statement stat,
             String sql) {
         try {
-            stat.executeQuery(sql);
-            fail("Expected error: " + expectedErrorMessage);
+            stat.execute(sql);
+            fail("Expected error: " + expectedErrorCode);
         } catch (SQLException ex) {
-            assertStartsWith(ex.getMessage(), expectedErrorMessage);
+            assertEquals(expectedErrorCode, ex.getErrorCode());
         }
     }
 
@@ -1270,7 +1290,9 @@ public abstract class TestBase {
      */
     protected void assertEquals(Integer expected, Integer actual) {
         if (expected == null || actual == null) {
-            assertTrue(expected == null && actual == null);
+            if (expected != actual) {
+                assertEquals("" + expected, "" + actual);
+            }
         } else {
             assertEquals(expected.intValue(), actual.intValue());
         }
@@ -1579,4 +1601,8 @@ public abstract class TestBase {
         throw (E) e;
     }
 
+    protected String getTestName() {
+        return getClass().getSimpleName();
+    }
+
 }
