@@ -36,6 +36,7 @@ public class TestCompatibility extends TestBase {
     public void test() throws SQLException {
         deleteDb("compatibility");
 
+        testOnDuplicateKey();
         testCaseSensitiveIdentifiers();
         testKeyAsColumnInMySQLMode();
 
@@ -55,6 +56,19 @@ public class TestCompatibility extends TestBase {
         deleteDb("compatibility");
     }
 
+    private void testOnDuplicateKey() throws SQLException {
+        Connection c = getConnection("compatibility;MODE=MYSQL");
+        Statement stat = c.createStatement();
+        stat.execute("set mode mysql");
+        stat.execute("create schema s2");
+        stat.execute("create table s2.test(id int primary key, name varchar(255))");
+        stat.execute("insert into s2.test(id, name) values(1, 'a')");
+        stat.execute("insert into s2.test(id, name) values(1, 'b') " +
+                "on duplicate key update name = values(name)");
+        stat.execute("drop schema s2");
+        c.close();
+    }
+
     private void testKeyAsColumnInMySQLMode() throws SQLException {
         Connection c = getConnection("compatibility;MODE=MYSQL");
         Statement stat = c.createStatement();
@@ -385,6 +399,10 @@ public class TestCompatibility extends TestBase {
         rs.next();
         assertEquals(10, rs.getInt(1));
         rs.close();
+        rs = stat.executeQuery("SELECT X FROM (SELECT CONVERT(INT, '10') AS X)");
+        rs.next();
+        assertEquals(10, rs.getInt(1));
+        rs.close();
 
         // make sure we're ignoring the index part of the statement
         rs = stat.executeQuery("select * from test (index table1_index)");
@@ -418,6 +436,8 @@ public class TestCompatibility extends TestBase {
         res.next();
         assertEquals("2", res.getString(1));
         assertFalse(res.next());
+        conn.close();
+
         // test isolation-clause
         conn = getConnection("compatibility;MODE=DB2");
         stat = conn.createStatement();
