@@ -27,6 +27,8 @@ import java.sql.SQLException;
 import java.sql.Statement;
 import java.sql.Timestamp;
 import java.sql.Types;
+import java.text.ParseException;
+import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Calendar;
 import java.util.Currency;
@@ -36,17 +38,21 @@ import java.util.Locale;
 import java.util.Properties;
 import java.util.TimeZone;
 import java.util.UUID;
-
 import org.h2.api.Aggregate;
 import org.h2.api.AggregateFunction;
 import org.h2.api.ErrorCode;
 import org.h2.engine.Constants;
+import org.h2.jdbc.JdbcSQLException;
+import org.h2.message.DbException;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
+import org.h2.test.ap.TestAnnotationProcessor;
 import org.h2.tools.SimpleResultSet;
+import org.h2.util.DateTimeUtils;
 import org.h2.util.IOUtils;
 import org.h2.util.New;
 import org.h2.util.StringUtils;
+import org.h2.util.ToDateParser;
 import org.h2.value.Value;
 
 /**
@@ -70,6 +76,9 @@ public class TestFunctions extends TestBase implements AggregateFunction {
     @Override
     public void test() throws Exception {
         deleteDb("functions");
+        testToDate();
+        testToDateException();
+        testAddMonths();
         testDataType();
         testVersion();
         testFunctionTable();
@@ -103,6 +112,7 @@ public class TestFunctions extends TestBase implements AggregateFunction {
         testTranslate();
         testGenerateSeries();
         testFileWrite();
+        testAnnotationProcessorsOutput();
 
         deleteDb("functions");
     }
@@ -1163,7 +1173,7 @@ public class TestFunctions extends TestBase implements AggregateFunction {
         call.execute();
         assertEquals(Integer[].class.getName(), call.getArray(1).getArray()
                 .getClass().getName());
-        assertEquals(new Integer[] { 2, 1 }, (Integer[]) call.getObject(1));
+        assertEquals(new Integer[]{2, 1}, (Integer[]) call.getObject(1));
 
         stat.execute("drop alias array_test");
 
@@ -1272,6 +1282,166 @@ public class TestFunctions extends TestBase implements AggregateFunction {
                 String.format("SELECT ORA_HASH('%s', 0, 0) FROM DUAL", testStr));
     }
 
+    private void testToDateException() {
+        try {
+            ToDateParser.toDate("1979-ThisWillFail-12", "YYYY-MM-DD");
+        } catch (Exception e) {
+            assertEquals(DbException.class.getSimpleName(), e.getClass().getSimpleName());
+        }
+    }
+
+    private void testToDate() throws ParseException {
+
+        final int month = Calendar.getInstance().get(Calendar.MONTH);
+
+        Date date = null;
+        date = new SimpleDateFormat("yyyy-MM-dd").parse("1979-11-12");
+        assertEquals(date, ToDateParser.toDate("1979-11-12", "YYYY-MM-DD"));
+        assertEquals(date, ToDateParser.toDate("1979/11/12", "YYYY/MM/DD"));
+        assertEquals(date, ToDateParser.toDate("1979,11,12", "YYYY,MM,DD"));
+        assertEquals(date, ToDateParser.toDate("1979.11.12", "YYYY.MM.DD"));
+        assertEquals(date, ToDateParser.toDate("1979;11;12", "YYYY;MM;DD"));
+        assertEquals(date, ToDateParser.toDate("1979:11:12", "YYYY:MM:DD"));
+
+        date = new SimpleDateFormat("yyyy").parse("1979");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("1979", "YYYY"));
+        assertEquals(date, ToDateParser.toDate("1979 AD", "YYYY AD"));
+        assertEquals(date, ToDateParser.toDate("1979 A.D.", "YYYY A.D."));
+        assertEquals(date, ToDateParser.toDate("1979 A.D.", "YYYY BC"));
+        assertEquals(date, ToDateParser.toDate("1979", "IYYY"));
+        assertEquals(date, ToDateParser.toDate("+1979", "SYYYY"));
+        assertEquals(date, ToDateParser.toDate("79", "RRRR"));
+
+        date = new SimpleDateFormat("yyyy-mm").parse("1970-12");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("12", "MI"));
+
+        date = new SimpleDateFormat("yyyy-MM").parse("1970-11");
+        assertEquals(date, ToDateParser.toDate("11", "MM"));
+        assertEquals(date, ToDateParser.toDate("11", "Mm"));
+        assertEquals(date, ToDateParser.toDate("11", "mM"));
+        assertEquals(date, ToDateParser.toDate("11", "mm"));
+        assertEquals(date, ToDateParser.toDate("XI", "RM"));
+
+        date = new SimpleDateFormat("yyyy").parse("9");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("9", "Y"));
+        assertEquals(date, ToDateParser.toDate("9", "I"));
+        date = new SimpleDateFormat("yyyy").parse("79");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("79", "YY"));
+        assertEquals(date, ToDateParser.toDate("79", "IY"));
+
+        date = new SimpleDateFormat("yyyy").parse("979");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("979", "YYY"));
+        assertEquals(date, ToDateParser.toDate("979", "IYY"));
+
+        // Gregorian calendar does not have a year 0.
+        // 0 = 0001 BC, -1 = 0002 BC, ... so we adjust
+        date = new SimpleDateFormat("yyy").parse("-99");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("0100 BC", "YYYY BC"));
+        assertEquals(date, ToDateParser.toDate("0100 B.C.", "YYYY B.C."));
+        assertEquals(date, ToDateParser.toDate("100 BC", "YYY BC"));
+        assertEquals(date, ToDateParser.toDate("-0100", "SYYYY"));
+        assertEquals(date, ToDateParser.toDate("-0100", "YYYY"));
+
+        // Gregorian calendar does not have a year 0.
+        // 0 = 0001 BC, -1 = 0002 BC, ... so we adjust
+        date = new SimpleDateFormat("y").parse("0");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("01 BC", "YY BC"));
+        assertEquals(date, ToDateParser.toDate("1 BC", "Y BC"));
+
+        date = new SimpleDateFormat("hh:mm:ss").parse("08:12:00");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("08:12 AM", "HH:MI AM"));
+        assertEquals(date, ToDateParser.toDate("08:12 A.M.", "HH:MI A.M."));
+        assertEquals(date, ToDateParser.toDate("08:12", "HH24:MI"));
+
+        date = new SimpleDateFormat("hh:mm:ss").parse("08:12:00");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("08:12", "HH:MI"));
+        assertEquals(date, ToDateParser.toDate("08:12", "HH12:MI"));
+
+        date = new SimpleDateFormat("hh:mm:ss").parse("08:12:34");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("08:12:34", "HH:MI:SS"));
+
+        date = new SimpleDateFormat("ss").parse("34");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("34", "SS"));
+
+        date = new SimpleDateFormat("yyyy hh:mm:ss").parse("1970 08:12:34");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("29554", "SSSSS"));
+
+        date = new SimpleDateFormat("yyyy hh:mm:ss SSS").parse("1970 08:12:34 550");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("08:12:34 550", "HH:MI:SS FF"));
+        assertEquals(date, ToDateParser.toDate("08:12:34 55", "HH:MI:SS FF2"));
+
+        date = new SimpleDateFormat("hh:mm:ss").parse("14:04:00");
+        setMonth(date, month);
+        assertEquals(date, ToDateParser.toDate("02:04 P.M.", "HH:MI p.M."));
+        assertEquals(date, ToDateParser.toDate("02:04 PM", "HH:MI PM"));
+
+        date = new SimpleDateFormat("yyyy-MM-dd").parse("1970-12-12");
+        // does not work in all timezones
+        // assertEquals(date, ToDateParser.toDate("12", "DD"));
+
+        date = new SimpleDateFormat("yyyy-MM-dd").parse("1970-11-12");
+        assertEquals(date, ToDateParser.toDate("316", "DDD"));
+        assertEquals(date, ToDateParser.toDate("316", "DdD"));
+        assertEquals(date, ToDateParser.toDate("316", "dDD"));
+        assertEquals(date, ToDateParser.toDate("316", "ddd"));
+
+        date = new SimpleDateFormat("yyyy-MM-dd").parse("2013-01-29");
+        assertEquals(date, ToDateParser.toDate("113029", "J"));
+    }
+
+    private static void setMonth(Date date, int month) {
+        Calendar c = Calendar.getInstance();
+        c.setTime(date);
+        c.set(Calendar.MONTH, month);
+        date.setTime(c.getTimeInMillis());
+    }
+
+    private void testAddMonths() throws ParseException {
+        Timestamp date;
+        Timestamp expected;
+
+        // 01-Aug-03 + 3 months = 01-Nov-03
+        date = new Timestamp(
+                new SimpleDateFormat("yyyy-MM-dd").parse("2003-08-01").getTime());
+        expected = new Timestamp(
+                new SimpleDateFormat("yyyy-MM-dd").parse("2003-11-01").getTime());
+        assertEquals(expected, DateTimeUtils.addMonths(new Timestamp(date.getTime()), 3));
+
+        // 31-Jan-03 + 1 month = 28-Feb-2003
+        date = new Timestamp(
+                new SimpleDateFormat("yyyy-MM-dd").parse("2003-01-31").getTime());
+        expected = new Timestamp(
+                new SimpleDateFormat("yyyy-MM-dd").parse("2003-02-28").getTime());
+        assertEquals(expected, DateTimeUtils.addMonths(new Timestamp(date.getTime()), 1));
+
+        // 21-Aug-2003 - 3 months = 21-May-2003
+        date = new Timestamp(
+                new SimpleDateFormat("yyyy-MM-dd").parse("2003-08-21").getTime());
+        expected = new Timestamp(
+                new SimpleDateFormat("yyyy-MM-dd").parse("2003-05-21").getTime());
+        assertEquals(expected, DateTimeUtils.addMonths(new Timestamp(date.getTime()), -3));
+
+        // 21-Aug-2003 00:00:00:333 - 3 months = 21-May-2003 00:00:00:333
+        date = new Timestamp(
+                new SimpleDateFormat("yyyy-MM-dd SSS").parse("2003-08-21 333").getTime());
+        expected = new Timestamp(
+                new SimpleDateFormat("yyyy-MM-dd SSS").parse("2003-05-21 333").getTime());
+        assertEquals(expected, DateTimeUtils.addMonths(new Timestamp(date.getTime()), -3));
+    }
+
     private void testToCharFromDateTime() throws SQLException {
         deleteDb("functions");
         Connection conn = getConnection("functions");
@@ -1699,6 +1869,24 @@ public class TestFunctions extends TestBase implements AggregateFunction {
         assertThrows(ErrorCode.INVALID_TO_CHAR_FORMAT, stat,
                 "SELECT TO_CHAR(123.45, 'q999.99') FROM DUAL");
 
+        // ISSUE-115
+        assertResult("0.123", stat, "select to_char(0.123, 'FM0.099') from dual;");
+        assertResult("1.123", stat, "select to_char(1.1234, 'FM0.099') from dual;");
+        assertResult("1.1234", stat, "select to_char(1.1234, 'FM0.0999') from dual;");
+        assertResult("1.023", stat, "select to_char(1.023, 'FM0.099') from dual;");
+        assertResult("0.012", stat, "select to_char(0.012, 'FM0.099') from dual;");
+        assertResult("0.123", stat, "select to_char(0.123, 'FM0.099') from dual;");
+        assertResult("0.001", stat, "select to_char(0.001, 'FM0.099') from dual;");
+        assertResult("0.001", stat, "select to_char(0.0012, 'FM0.099') from dual;");
+        assertResult("0.002", stat, "select to_char(0.0019, 'FM0.099') from dual;");
+        assertResult("0.0", stat, "select to_char(0, 'FM0D099') from dual;");
+        assertResult("0.00", stat, "select to_char(0., 'FM0D009') from dual;");
+        assertResult("0.", stat, "select to_char(0, 'FM0D9') from dual;");
+        assertResult("0.0", stat, "select to_char(0.0, 'FM0D099') from dual;");
+        assertResult("0.00", stat, "select to_char(0.00, 'FM0D009') from dual;");
+        assertResult("0.00", stat, "select to_char(0, 'FM0D009') from dual;");
+        assertResult("0.0", stat, "select to_char(0, 'FM0D09') from dual;");
+        assertResult("0.0", stat, "select to_char(0, 'FM0D0') from dual;");
         conn.close();
     }
 
@@ -1760,6 +1948,40 @@ public class TestFunctions extends TestBase implements AggregateFunction {
         conn.close();
     }
 
+    private void testAnnotationProcessorsOutput() throws SQLException {
+        try {
+            System.setProperty(TestAnnotationProcessor.MESSAGES_KEY, "WARNING,foo1|ERROR,foo2");
+            callCompiledFunction("test_annotation_processor_warn_and_error");
+            fail();
+        } catch (JdbcSQLException e) {
+            assertEquals(ErrorCode.SYNTAX_ERROR_1, e.getErrorCode());
+            assertContains(e.getMessage(), "foo1");
+            assertContains(e.getMessage(), "foo2");
+        } finally {
+            System.clearProperty(TestAnnotationProcessor.MESSAGES_KEY);
+        }
+    }
+
+    private void callCompiledFunction(String functionName) throws SQLException {
+        deleteDb("functions");
+        Connection conn = getConnection("functions");
+        Statement stat = conn.createStatement();
+        ResultSet rs;
+        stat.execute("create alias " + functionName + " AS "
+                + "$$ boolean " + functionName + "() "
+                + "{ return true; } $$;");
+
+        PreparedStatement stmt = conn.prepareStatement(
+                "select " + functionName + "() from dual");
+        rs = stmt.executeQuery();
+        rs.next();
+        assertEquals(Boolean.class.getName(), rs.getObject(1).getClass().getName());
+
+        stat.execute("drop alias " + functionName + "");
+
+        conn.close();
+    }
+
     private void assertCallResult(String expected, Statement stat, String sql)
             throws SQLException {
         ResultSet rs = stat.executeQuery("CALL " + sql);
