@@ -62,6 +62,8 @@ public class TestOptimizations extends TestBase {
         testNestedInSelect();
         testInSelectJoin();
         testMinMaxNullOptimization();
+        testUseCoveringIndex();
+        // testUseIndexWhenAllColumnsNotInOrderBy();
         if (config.networked) {
             return;
         }
@@ -669,6 +671,25 @@ public class TestOptimizations extends TestBase {
         conn.close();
     }
 
+    private void testUseIndexWhenAllColumnsNotInOrderBy() throws SQLException {
+        deleteDb("optimizations");
+        Connection conn = getConnection("optimizations");
+        Statement stat = conn.createStatement();
+        stat.execute("create table test(id int primary key, account int, tx int)");
+        stat.execute("insert into test select x, x*100, x from system_range(1, 10000)");
+        stat.execute("analyze sample_size 5");
+        stat.execute("create unique index idx_test_account_tx on test(account, tx desc)");
+        ResultSet rs;
+        rs = stat.executeQuery("explain analyze " +
+                "select tx from test " +
+                "where account=22 and tx<9999999 " +
+                "order by tx desc limit 25");
+        rs.next();
+        String plan = rs.getString(1);
+        assertContains(plan, "index sorted");
+        conn.close();
+    }
+
     private void testDistinctOptimization() throws SQLException {
         deleteDb("optimizations");
         Connection conn = getConnection("optimizations");
@@ -1007,4 +1028,37 @@ public class TestOptimizations extends TestBase {
 
         conn.close();
     }
+
+    private void testUseCoveringIndex() throws SQLException {
+        deleteDb("optimizations");
+        Connection conn = getConnection("optimizations");
+        Statement stat = conn.createStatement();
+        stat.execute("CREATE TABLE TABLE_A(id IDENTITY PRIMARY KEY NOT NULL, " +
+                "name VARCHAR NOT NULL, active BOOLEAN DEFAULT TRUE, " +
+                "UNIQUE KEY TABLE_A_UK (name) )");
+        stat.execute("CREATE TABLE TABLE_B(id IDENTITY PRIMARY KEY NOT NULL,  " +
+                "TABLE_a_id BIGINT NOT NULL,  createDate TIMESTAMP DEFAULT NOW(), " +
+                "UNIQUE KEY TABLE_B_UK (table_a_id, createDate), " +
+                "FOREIGN KEY (table_a_id) REFERENCES TABLE_A(id) )");
+        stat.execute("INSERT INTO TABLE_A (name)  SELECT 'package_' || CAST(X as VARCHAR) " +
+                "FROM SYSTEM_RANGE(1, 100)  WHERE X <= 100");
+        stat.execute("INSERT INTO TABLE_B (table_a_id, createDate)  SELECT " +
+                "CASE WHEN table_a_id = 0 THEN 1 ELSE table_a_id END, createDate " +
+                "FROM ( SELECT ROUND((RAND() * 100)) AS table_a_id, " +
+                "DATEADD('SECOND', X, NOW()) as createDate FROM SYSTEM_RANGE(1, 50000) " +
+                "WHERE X < 50000  )");
+        stat.execute("CREATE INDEX table_b_idx ON table_b(table_a_id, id)");
+        stat.execute("ANALYZE");
+
+        ResultSet rs = stat.executeQuery("EXPLAIN ANALYZE SELECT MAX(b.id) as id " +
+                "FROM table_b b JOIN table_a a ON b.table_a_id = a.id GROUP BY b.table_a_id " +
+                "HAVING A.ACTIVE = TRUE");
+        rs.next();
+        assertContains(rs.getString(1), "/* PUBLIC.TABLE_B_IDX: TABLE_A_ID = A.ID */");
+
+        rs = stat.executeQuery("EXPLAIN ANALYZE SELECT MAX(id) FROM table_b GROUP BY table_a_id");
+        rs.next();
+        assertContains(rs.getString(1), "/* PUBLIC.TABLE_B_IDX");
+        conn.close();
+    }
 }
