@@ -13,7 +13,6 @@ import java.math.BigDecimal;
 import java.net.URL;
 import java.sql.Array;
 import java.sql.Connection;
-import java.sql.Date;
 import java.sql.ParameterMetaData;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
@@ -21,19 +20,12 @@ import java.sql.ResultSetMetaData;
 import java.sql.RowId;
 import java.sql.SQLException;
 import java.sql.Statement;
-import java.sql.Time;
 import java.sql.Timestamp;
 import java.sql.Types;
-import java.util.Calendar;
-import java.util.GregorianCalendar;
-import java.util.SimpleTimeZone;
-import java.util.TimeZone;
 import java.util.UUID;
-
 import org.h2.api.ErrorCode;
 import org.h2.api.Trigger;
 import org.h2.test.TestBase;
-import org.h2.util.DateTimeUtils;
 import org.h2.util.Task;
 
 /**
@@ -59,7 +51,6 @@ public class TestPreparedStatement extends TestBase {
         testUnwrap(conn);
         testUnsupportedOperations(conn);
         testChangeType(conn);
-        testDateTimeTimestampWithCalendar(conn);
         testCallTablePrepared(conn);
         testValues(conn);
         testToString(conn);
@@ -87,11 +78,14 @@ public class TestPreparedStatement extends TestBase {
         testSubquery(conn);
         testObject(conn);
         testIdentity(conn);
+        testBatchGeneratedKeys(conn);
         testDataTypes(conn);
         testGetMoreResults(conn);
         testBlob(conn);
         testClob(conn);
         testParameterMetaData(conn);
+        testColumnMetaDataWithEquals(conn);
+        testColumnMetaDataWithIn(conn);
         conn.close();
         testPreparedStatementWithLiteralsNone();
         deleteDb("preparedStatement");
@@ -167,108 +161,6 @@ public class TestPreparedStatement extends TestBase {
         prep.executeQuery();
     }
 
-    private void testDateTimeTimestampWithCalendar(Connection conn)
-            throws SQLException {
-        Statement stat = conn.createStatement();
-        stat.execute("create table ts(x timestamp primary key)");
-        stat.execute("create table t(x time primary key)");
-        stat.execute("create table d(x date)");
-        Calendar utcCalendar = new GregorianCalendar(new SimpleTimeZone(0, "Z"));
-        TimeZone old = TimeZone.getDefault();
-        DateTimeUtils.resetCalendar();
-        TimeZone.setDefault(TimeZone.getTimeZone("PST"));
-        try {
-            Timestamp ts1 = Timestamp.valueOf("2010-03-13 18:15:00");
-            Time t1 = new Time(ts1.getTime());
-            Date d1 = new Date(ts1.getTime());
-            // when converted to UTC, this is 03:15, which doesn't actually
-            // exist because of summer time change at that day
-            Timestamp ts2 = Timestamp.valueOf("2010-03-13 19:15:00");
-            Time t2 = new Time(ts2.getTime());
-            Date d2 = new Date(ts2.getTime());
-            PreparedStatement prep;
-            ResultSet rs;
-            prep = conn.prepareStatement("insert into ts values(?)");
-            prep.setTimestamp(1, ts1, utcCalendar);
-            prep.execute();
-            prep.setTimestamp(1, ts2, utcCalendar);
-            prep.execute();
-            prep = conn.prepareStatement("insert into t values(?)");
-            prep.setTime(1, t1, utcCalendar);
-            prep.execute();
-            prep.setTime(1, t2, utcCalendar);
-            prep.execute();
-            prep = conn.prepareStatement("insert into d values(?)");
-            prep.setDate(1, d1, utcCalendar);
-            prep.execute();
-            prep.setDate(1, d2, utcCalendar);
-            prep.execute();
-            rs = stat.executeQuery("select * from ts order by x");
-            rs.next();
-            assertEquals("2010-03-14 02:15:00.0",
-                    rs.getString(1));
-            assertEquals("2010-03-13 18:15:00.0",
-                    rs.getTimestamp(1, utcCalendar).toString());
-            assertEquals("2010-03-14 03:15:00.0",
-                    rs.getTimestamp(1).toString());
-            assertEquals("2010-03-14 02:15:00.0",
-                    rs.getString("x"));
-            assertEquals("2010-03-13 18:15:00.0",
-                    rs.getTimestamp("x", utcCalendar).toString());
-            assertEquals("2010-03-14 03:15:00.0",
-                    rs.getTimestamp("x").toString());
-            rs.next();
-            assertEquals("2010-03-14 03:15:00.0",
-                    rs.getString(1));
-            assertEquals("2010-03-13 19:15:00.0",
-                    rs.getTimestamp(1, utcCalendar).toString());
-            assertEquals("2010-03-14 03:15:00.0",
-                    rs.getTimestamp(1).toString());
-            assertEquals("2010-03-14 03:15:00.0",
-                    rs.getString("x"));
-            assertEquals("2010-03-13 19:15:00.0",
-                    rs.getTimestamp("x", utcCalendar).toString());
-            assertEquals("2010-03-14 03:15:00.0",
-                    rs.getTimestamp("x").toString());
-            rs = stat.executeQuery("select * from t order by x");
-            rs.next();
-            assertEquals("02:15:00", rs.getString(1));
-            assertEquals("18:15:00", rs.getTime(1, utcCalendar).toString());
-            assertEquals("02:15:00", rs.getTime(1).toString());
-            assertEquals("02:15:00", rs.getString("x"));
-            assertEquals("18:15:00", rs.getTime("x", utcCalendar).toString());
-            assertEquals("02:15:00", rs.getTime("x").toString());
-            rs.next();
-            assertEquals("03:15:00", rs.getString(1));
-            assertEquals("19:15:00", rs.getTime(1, utcCalendar).toString());
-            assertEquals("03:15:00", rs.getTime(1).toString());
-            assertEquals("03:15:00", rs.getString("x"));
-            assertEquals("19:15:00", rs.getTime("x", utcCalendar).toString());
-            assertEquals("03:15:00", rs.getTime("x").toString());
-            rs = stat.executeQuery("select * from d order by x");
-            rs.next();
-            assertEquals("2010-03-14", rs.getString(1));
-            assertEquals("2010-03-13", rs.getDate(1, utcCalendar).toString());
-            assertEquals("2010-03-14", rs.getDate(1).toString());
-            assertEquals("2010-03-14", rs.getString("x"));
-            assertEquals("2010-03-13", rs.getDate("x", utcCalendar).toString());
-            assertEquals("2010-03-14", rs.getDate("x").toString());
-            rs.next();
-            assertEquals("2010-03-14", rs.getString(1));
-            assertEquals("2010-03-13", rs.getDate(1, utcCalendar).toString());
-            assertEquals("2010-03-14", rs.getDate(1).toString());
-            assertEquals("2010-03-14", rs.getString("x"));
-            assertEquals("2010-03-13", rs.getDate("x", utcCalendar).toString());
-            assertEquals("2010-03-14", rs.getDate("x").toString());
-        } finally {
-            TimeZone.setDefault(old);
-            DateTimeUtils.resetCalendar();
-        }
-        stat.execute("drop table ts");
-        stat.execute("drop table t");
-        stat.execute("drop table d");
-    }
-
     private static void testCallTablePrepared(Connection conn) throws SQLException {
         PreparedStatement prep = conn.prepareStatement("call table(x int = (1))");
         prep.executeQuery();
@@ -612,11 +504,10 @@ public class TestPreparedStatement extends TestBase {
 
     private void testScopedGeneratedKey(Connection conn) throws SQLException {
         Statement stat = conn.createStatement();
-        Trigger t = new SequenceTrigger();
         stat.execute("create table test(id identity)");
         stat.execute("create sequence seq start with 1000");
         stat.execute("create trigger test_ins after insert on test call \"" +
-                t.getClass().getName() + "\"");
+                SequenceTrigger.class.getName() + "\"");
         stat.execute("insert into test values(null)");
         ResultSet rs = stat.getGeneratedKeys();
         rs.next();
@@ -1023,6 +914,7 @@ public class TestPreparedStatement extends TestBase {
         prep.executeUpdate();
         assertFalse(prep.getMoreResults());
         assertEquals(-1, prep.getUpdateCount());
+        stat.execute("DROP TABLE TEST");
     }
 
     private void testObject(Connection conn) throws SQLException {
@@ -1156,6 +1048,29 @@ public class TestPreparedStatement extends TestBase {
         assertFalse(rs.next());
 
         stat.execute("DROP TABLE TEST");
+        stat.execute("DROP SEQUENCE SEQ");
+    }
+
+    private void testBatchGeneratedKeys(Connection conn) throws SQLException {
+        Statement stat = conn.createStatement();
+        stat.execute("CREATE SEQUENCE SEQ");
+        stat.execute("CREATE TABLE TEST(ID INT)");
+        PreparedStatement prep = conn.prepareStatement(
+                "INSERT INTO TEST VALUES(NEXT VALUE FOR SEQ)");
+        prep.addBatch();
+        prep.addBatch();
+        prep.addBatch();
+        prep.executeBatch();
+        ResultSet keys = prep.getGeneratedKeys();
+        keys.next();
+        assertEquals(1, keys.getLong(1));
+        keys.next();
+        assertEquals(2, keys.getLong(1));
+        keys.next();
+        assertEquals(3, keys.getLong(1));
+        assertFalse(keys.next());
+        stat.execute("DROP TABLE TEST");
+        stat.execute("DROP SEQUENCE SEQ");
     }
 
     private int getLength() {
@@ -1362,4 +1277,33 @@ public class TestPreparedStatement extends TestBase {
         assertTrue(!rs.next());
     }
 
+    private void testColumnMetaDataWithEquals(Connection conn)
+            throws SQLException {
+        Statement stmt = conn.createStatement();
+        stmt.execute("CREATE TABLE TEST( id INT, someColumn INT )");
+        PreparedStatement ps = conn
+                .prepareStatement("INSERT INTO TEST VALUES(?,?)");
+        ps.setInt(1, 0);
+        ps.setInt(2, 999);
+        ps.execute();
+        ps = conn.prepareStatement("SELECT * FROM TEST WHERE someColumn = ?");
+        assertEquals(Types.INTEGER,
+                ps.getParameterMetaData().getParameterType(1));
+        stmt.execute("DROP TABLE TEST");
+    }
+
+    private void testColumnMetaDataWithIn(Connection conn) throws SQLException {
+        Statement stmt = conn.createStatement();
+        stmt.execute("CREATE TABLE TEST( id INT, someColumn INT )");
+        PreparedStatement ps = conn
+                .prepareStatement("INSERT INTO TEST VALUES( ? , ? )");
+        ps.setInt(1, 0);
+        ps.setInt(2, 999);
+        ps.execute();
+        ps = conn
+                .prepareStatement("SELECT * FROM TEST WHERE someColumn IN (?,?)");
+        assertEquals(Types.INTEGER,
+                ps.getParameterMetaData().getParameterType(1));
+        stmt.execute("DROP TABLE TEST");
+    }
 }
