@@ -32,7 +32,6 @@ import java.sql.Timestamp;
 import java.sql.Types;
 import java.util.Calendar;
 import java.util.Collections;
-import java.util.Locale;
 import java.util.TimeZone;
 
 import org.h2.api.ErrorCode;
@@ -70,7 +69,6 @@ public class TestResultSet extends TestBase {
         testInsertRowWithUpdatableResultSetDefault();
         testBeforeFirstAfterLast();
         testParseSpecialValues();
-        testSpecialLocale();
         testSubstringPrecision();
         testSubstringDataType();
         testColumnLabelColumnName();
@@ -179,7 +177,7 @@ public class TestResultSet extends TestBase {
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, rs).
                 getCursorName();
         assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, rs).
-                setFetchDirection(ResultSet.FETCH_FORWARD);
+                setFetchDirection(ResultSet.FETCH_REVERSE);
     }
 
     private void testAmbiguousColumnNames() throws SQLException {
@@ -347,26 +345,6 @@ public class TestResultSet extends TestBase {
         assertTrue(expected.equals(o));
     }
 
-    private void testSpecialLocale() throws SQLException {
-        Locale old = Locale.getDefault();
-        try {
-            // when using Turkish as the default locale, "i".toUpperCase() is
-            // not "I"
-            Locale.setDefault(new Locale("tr"));
-            stat.execute("create table test(I1 int, i2 int, b int, c int, d int) " +
-                    "as select 1, 1, 1, 1, 1");
-            ResultSet rs = stat.executeQuery("select * from test");
-            rs.next();
-            rs.getString("I1");
-            rs.getString("i1");
-            rs.getString("I2");
-            rs.getString("i2");
-            stat.execute("drop table test");
-        } finally {
-            Locale.setDefault(old);
-        }
-    }
-
     private void testSubstringDataType() throws SQLException {
         ResultSet rs = stat.executeQuery("select substr(x, 1, 1) from dual");
         rs.next();
@@ -418,6 +396,7 @@ public class TestResultSet extends TestBase {
         int a = stat.getFetchSize();
         int b = rs.getFetchSize();
         assertEquals(a, b);
+        rs.setFetchDirection(ResultSet.FETCH_FORWARD);
         rs.setFetchSize(b + 1);
         b = rs.getFetchSize();
         assertEquals(a + 1, b);
