@@ -10,6 +10,7 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map.Entry;
 import java.util.Random;
+
 import org.h2.mvstore.cache.CacheLongKeyLIRS;
 import org.h2.test.TestBase;
 import org.h2.util.New;
@@ -34,7 +35,6 @@ public class TestCacheLongKeyLIRS extends TestBase {
     }
 
     private void testCache() {
-        testResize();
         testRandomSmallCache();
         testEdgeCases();
         testSize();
@@ -43,25 +43,11 @@ public class TestCacheLongKeyLIRS extends TestBase {
         testPruneStack();
         testLimitHot();
         testLimitNonResident();
+        testLimitMemory();
         testScanResistance();
         testRandomOperations();
     }
 
-    private void testResize() {
-        // cache with 100 memory, average memory 10
-        // (that means 10 entries)
-        CacheLongKeyLIRS<Integer> t1 =
-                new CacheLongKeyLIRS<Integer>(100, 10, 1, 0);
-        // another cache with more entries
-        CacheLongKeyLIRS<Integer> t2 =
-                new CacheLongKeyLIRS<Integer>(100, 1, 1, 0);
-        for (int i = 0; i < 200; i++) {
-            t1.put(i, i, 1);
-            t2.put(i, i, 1);
-        }
-        assertEquals(toString(t2), toString(t1));
-    }
-
     private static void testRandomSmallCache() {
         Random r = new Random(1);
         for (int i = 0; i < 10000; i++) {
@@ -92,9 +78,9 @@ public class TestCacheLongKeyLIRS extends TestBase {
     private void testEdgeCases() {
         CacheLongKeyLIRS<Integer> test = createCache(1);
         test.put(1, 10, 100);
-        assertEquals(10, test.get(1).intValue());
+        assertEquals(0, test.size());
         try {
-            test.put(1,  null, 100);
+            test.put(1, null, 100);
             fail();
         } catch (IllegalArgumentException e) {
             // expected
@@ -105,12 +91,6 @@ public class TestCacheLongKeyLIRS extends TestBase {
         } catch (IllegalArgumentException e) {
             // expected
         }
-        try {
-            test.setAverageMemory(0);
-            fail();
-        } catch (IllegalArgumentException e) {
-            // expected
-        }
     }
 
     private void testSize() {
@@ -124,38 +104,39 @@ public class TestCacheLongKeyLIRS extends TestBase {
         verifyMapSize(769, 2048);
 
         CacheLongKeyLIRS<Integer> test;
-        test = createCache(3, 10);
-        test.put(0, 0, 9);
-        test.put(1, 10, 9);
-        test.put(2, 20, 9);
-        test.put(3, 30, 9);
-        test.put(4, 40, 9);
-
-        test = createCache(1, 1);
-        test.put(1, 10);
-        test.put(0, 0);
-        test.get(0);
 
         test = createCache(1000);
         for (int j = 0; j < 2000; j++) {
             test.put(j, j);
         }
         // for a cache of size 1000,
-        // there are 62 cold entries (about 6.25%).
-//        assertEquals(62, test.size() - test.sizeHot());
+        // there are 63 cold entries (about 6.25%).
+        assertEquals(63, test.size() - test.sizeHot());
         // at most as many non-resident elements
         // as there are entries in the stack
-//        assertEquals(968, test.sizeNonResident());
+        assertEquals(1000, test.size());
+        assertEquals(1000, test.sizeNonResident());
     }
 
     private void verifyMapSize(int elements, int expectedMapSize) {
         CacheLongKeyLIRS<Integer> test;
         test = createCache(elements - 1);
-        assertTrue(test.sizeMapArray() < expectedMapSize);
+        for (int i = 0; i < elements - 1; i++) {
+            test.put(i, i * 10);
+        }
+        assertTrue(test.sizeMapArray() + "<" + expectedMapSize,
+                test.sizeMapArray() < expectedMapSize);
         test = createCache(elements);
+        for (int i = 0; i < elements + 1; i++) {
+            test.put(i, i * 10);
+        }
         assertEquals(expectedMapSize, test.sizeMapArray());
-        test = createCache(elements * 100, 100);
-        assertEquals(expectedMapSize, test.sizeMapArray());
+        test = createCache(elements * 2);
+        for (int i = 0; i < elements * 2; i++) {
+            test.put(i, i * 10);
+        }
+        assertTrue(test.sizeMapArray() + ">" + expectedMapSize,
+                test.sizeMapArray() > expectedMapSize);
     }
 
     private void testGetPutPeekRemove() {
@@ -295,7 +276,7 @@ public class TestCacheLongKeyLIRS extends TestBase {
     }
 
     private void testClear() {
-        CacheLongKeyLIRS<Integer> test = createCache(40, 10);
+        CacheLongKeyLIRS<Integer> test = createCache(40);
         for (int i = 0; i < 5; i++) {
             test.put(i, 10 * i, 9);
         }
@@ -311,7 +292,6 @@ public class TestCacheLongKeyLIRS extends TestBase {
             assertTrue(x >= 1 && x <= 4);
         }
         assertEquals(40,  test.getMaxMemory());
-        assertEquals(10, test.getAverageMemory());
         assertEquals(36,  test.getUsedMemory());
         assertEquals(4, test.size());
         assertEquals(3,  test.sizeHot());
@@ -322,20 +302,15 @@ public class TestCacheLongKeyLIRS extends TestBase {
         test.setMaxMemory(10);
         assertEquals(10, test.getMaxMemory());
         test.setMaxMemory(40);
-        test.setAverageMemory(1);
-        assertEquals(1, test.getAverageMemory());
-        test.setAverageMemory(10);
         verify(test, "mem: 36 stack: 4 3 2 1 cold: 4 non-resident: 0");
 
-        // putAll uses the average memory
         test.putAll(test.getMap());
-        verify(test, "mem: 40 stack: 4 3 2 1 cold: non-resident: 0");
+        verify(test, "mem: 4 stack: 4 3 2 1 cold: non-resident: 0");
 
         test.clear();
         verify(test, "mem: 0 stack: cold: non-resident:");
 
         assertEquals(40,  test.getMaxMemory());
-        assertEquals(10, test.getAverageMemory());
         assertEquals(0,  test.getUsedMemory());
         assertEquals(0, test.size());
         assertEquals(0,  test.sizeHot());
@@ -349,8 +324,8 @@ public class TestCacheLongKeyLIRS extends TestBase {
             test.put(i, 10 * i);
         }
         assertEquals(100, test.size());
-        assertEquals(99, test.sizeNonResident());
-        assertEquals(93, test.sizeHot());
+        assertEquals(200, test.sizeNonResident());
+        assertEquals(90, test.sizeHot());
     }
 
     private void testLimitNonResident() {
@@ -358,15 +333,33 @@ public class TestCacheLongKeyLIRS extends TestBase {
         for (int i = 0; i < 20; i++) {
             test.put(i, 10 * i);
         }
-        verify(test, "mem: 4 stack: 19 18 17 16 3 2 1 " +
-                "cold: 19 non-resident: 18 17 16");
+        verify(test, "mem: 4 stack: 19 18 17 16 15 14 13 12 11 10 3 2 1 " +
+                "cold: 19 non-resident: 18 17 16 15 14 13 12 11 10");
+    }
+
+    private void testLimitMemory() {
+        CacheLongKeyLIRS<Integer> test = createCache(4);
+        for (int i = 0; i < 5; i++) {
+            test.put(i, 10 * i, 1);
+        }
+        verify(test, "mem: 4 stack: 4 3 2 1 cold: 4 non-resident: 0");
+        assertTrue("" + test.getUsedMemory(), test.getUsedMemory() <= 4);
+        test.put(6, 60, 3);
+        verify(test, "mem: 4 stack: 6 4 3 cold: 6 non-resident: 2 1 4");
+        assertTrue("" + test.getUsedMemory(), test.getUsedMemory() <= 4);
+        test.put(7, 70, 3);
+        verify(test, "mem: 4 stack: 7 6 3 cold: 7 non-resident: 6 2 1");
+        assertTrue("" + test.getUsedMemory(), test.getUsedMemory() <= 4);
+        test.put(8, 80, 4);
+        verify(test, "mem: 4 stack: 8 cold: non-resident:");
+        assertTrue("" + test.getUsedMemory(), test.getUsedMemory() <= 4);
     }
 
     private void testScanResistance() {
         boolean log = false;
         int size = 20;
-        // cache size 11 (10 hot, 1 cold)
-        CacheLongKeyLIRS<Integer> test = createCache(size / 2 + 1);
+        // cache size 11 (10 hot, 2 cold)
+        CacheLongKeyLIRS<Integer> test = createCache(size / 2 + 2);
         // init the cache with some dummy entries
         for (int i = 0; i < size; i++) {
             test.put(-i, -i * 10);
@@ -402,10 +395,10 @@ public class TestCacheLongKeyLIRS extends TestBase {
             }
             verify(test, null);
         }
-        // ensure 0..9 are hot, 10..18 are not resident, 19 is cold
+        // ensure 0..9 are hot, 10..17 are not resident, 18..19 are cold
         for (int i = 0; i < size; i++) {
             Integer x = test.get(i);
-            if (i < size / 2 || i == size - 1) {
+            if (i < size / 2 || i == size - 1 || i == size - 2) {
                 assertTrue("i: " + i, x != null);
                 assertEquals(i * 10, x.intValue());
             } else {
@@ -504,13 +497,12 @@ public class TestCacheLongKeyLIRS extends TestBase {
         }
     }
 
-    private static <V> CacheLongKeyLIRS<V> createCache(int maxElements) {
-        return createCache(maxElements, 1);
-    }
-
-    private static <V> CacheLongKeyLIRS<V> createCache(int maxSize,
-            int averageSize) {
-        return new CacheLongKeyLIRS<V>(maxSize, averageSize, 1, 0);
+    private static <V> CacheLongKeyLIRS<V> createCache(int maxSize) {
+        CacheLongKeyLIRS.Config cc = new CacheLongKeyLIRS.Config();
+        cc.maxMemory = maxSize;
+        cc.segmentCount = 1;
+        cc.stackMoveDistance = 0;
+        return new CacheLongKeyLIRS<V>(cc);
     }
 
 }
