@@ -50,8 +50,6 @@ public class TestMVStore extends TestBase {
 
     @Override
     public void test() throws Exception {
-        FileUtils.deleteRecursive(getBaseDir(), true);
-        FileUtils.createDirectories(getBaseDir());
         testRemoveMapRollback();
         testProvidedFileStoreNotOpenedAndClosed();
         testVolatileMap();
@@ -127,7 +125,8 @@ public class TestMVStore extends TestBase {
         assertNull(map.get("1"));
         store.close();
 
-        String fileName = getBaseDir() + "/testRemoveMapRollback.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         store = new MVStore.Builder().
                 autoCommitDisabled().
                 fileName(fileName).
@@ -168,7 +167,8 @@ public class TestMVStore extends TestBase {
     }
 
     private void testVolatileMap() {
-        String fileName = getBaseDir() + "/testVolatile.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         MVStore store = new MVStore.Builder().
                 fileName(fileName).
                 open();
@@ -205,7 +205,8 @@ public class TestMVStore extends TestBase {
     }
 
     private void testCompressEmptyPage() {
-        String fileName = getBaseDir() + "/testDeletedMap.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         MVStore store = new MVStore.Builder().
                 cacheSize(100).fileName(fileName).
                 compress().
@@ -222,7 +223,8 @@ public class TestMVStore extends TestBase {
     }
 
     private void testCompressed() {
-        String fileName = getBaseDir() + "/testCompressed.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         long lastSize = 0;
         for (int level = 0; level <= 2; level++) {
             FileUtils.delete(fileName);
@@ -254,7 +256,8 @@ public class TestMVStore extends TestBase {
     }
 
     private void testFileFormatExample() {
-        String fileName = getBaseDir() + "/testFileFormatExample.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         MVStore s = MVStore.open(fileName);
         MVMap<Integer, String> map = s.openMap("data");
         for (int i = 0; i < 400; i++) {
@@ -270,7 +273,8 @@ public class TestMVStore extends TestBase {
     }
 
     private void testMaxChunkLength() {
-        String fileName = getBaseDir() + "/testMaxChunkLength.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         MVStore s = new MVStore.Builder().fileName(fileName).open();
         MVMap<Integer, byte[]> map = s.openMap("data");
         map.put(0, new byte[2 * 1024 * 1024]);
@@ -285,18 +289,19 @@ public class TestMVStore extends TestBase {
     }
 
     private void testCacheInfo() {
-        String fileName = getBaseDir() + "/testCloseMap.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         MVStore s = new MVStore.Builder().fileName(fileName).cacheSize(2).open();
         assertEquals(2, s.getCacheSize());
         MVMap<Integer, byte[]> map;
         map = s.openMap("data");
-        byte[] data = new byte[100 * 1024];
-        for (int i = 0; i < 30; i++) {
+        byte[] data = new byte[1024];
+        for (int i = 0; i < 1000; i++) {
             map.put(i, data);
             s.commit();
-            if (i < 5) {
+            if (i < 50) {
                 assertEquals(0, s.getCacheSizeUsed());
-            } else if (i > 20) {
+            } else if (i > 300) {
                 assertTrue(s.getCacheSizeUsed() >= 1);
             }
         }
@@ -355,7 +360,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testRemoveMap() throws Exception {
-        String fileName = getBaseDir() + "/testCloseMap.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = new MVStore.Builder().
             fileName(fileName).
@@ -418,7 +423,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testNewerWriteVersion() throws Exception {
-        String fileName = getBaseDir() + "/testNewerWriteVersion.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = new MVStore.Builder().
                 encryptionKey("007".toCharArray()).
@@ -429,16 +434,9 @@ public class TestMVStore extends TestBase {
         assertEquals("1", header.get("format").toString());
         header.put("formatRead", "1");
         header.put("format", "2");
+        forceWriteStoreHeader(s);
         MVMap<Integer, String> m = s.openMap("data");
-        // this is to ensure the file header is overwritten
-        // the header is written at least every 20 commits
-        for (int i = 0; i < 30; i++) {
-            m.put(0, "Hello World " + i);
-            s.commit();
-            if (i > 5) {
-                s.setRetentionTime(0);
-            }
-        }
+        forceWriteStoreHeader(s);
         m.put(0, "Hello World");
         s.close();
         try {
@@ -475,7 +473,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testCompactFully() throws Exception {
-        String fileName = getBaseDir() + "/testCompactFully.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = new MVStore.Builder().
                 fileName(fileName).
@@ -500,7 +498,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testBackgroundExceptionListener() throws Exception {
-        String fileName = getBaseDir() + "/testBackgroundExceptionListener.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s;
         final AtomicReference<Throwable> exRef =
@@ -516,28 +514,34 @@ public class TestMVStore extends TestBase {
 
                 }).
                 open();
-        s.setAutoCommitDelay(50);
+        s.setAutoCommitDelay(10);
         MVMap<Integer, String> m;
         m = s.openMap("data");
         s.getFileStore().getFile().close();
-        m.put(1, "Hello");
-        for (int i = 0; i < 200; i++) {
-            if (exRef.get() != null) {
-                break;
+        try {
+            m.put(1, "Hello");
+            for (int i = 0; i < 200; i++) {
+                if (exRef.get() != null) {
+                    break;
+                }
+                sleep(10);
             }
-            Thread.sleep(1);
+            Throwable e = exRef.get();
+            assertTrue(e != null);
+            assertEquals(DataUtils.ERROR_WRITING_FAILED,
+                    DataUtils.getErrorCode(e.getMessage()));
+        } catch (IllegalStateException e) {
+            // sometimes it is detected right away
+            assertEquals(DataUtils.ERROR_CLOSED,
+                    DataUtils.getErrorCode(e.getMessage()));
         }
-        Throwable e = exRef.get();
-        assertTrue(e != null);
-        assertEquals(DataUtils.ERROR_WRITING_FAILED,
-                DataUtils.getErrorCode(e.getMessage()));
 
         s.closeImmediately();
         FileUtils.delete(fileName);
     }
 
     private void testAtomicOperations() {
-        String fileName = getBaseDir() + "/testAtomicOperations.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s;
         MVMap<Integer, byte[]> m;
@@ -572,7 +576,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testWriteBuffer() {
-        String fileName = getBaseDir() + "/testWriteBuffer.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s;
         MVMap<Integer, byte[]> m;
@@ -616,8 +620,9 @@ public class TestMVStore extends TestBase {
         FileUtils.delete(fileName);
     }
 
-    private void testWriteDelay() throws InterruptedException {
-        String fileName = getBaseDir() + "/testWriteDelay.h3";
+    private void testWriteDelay() {
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         MVStore s;
         MVMap<Integer, String> m;
 
@@ -640,34 +645,33 @@ public class TestMVStore extends TestBase {
         s = new MVStore.Builder().
                 fileName(fileName).
                 open();
-        s.setAutoCommitDelay(1);
         m = s.openMap("data");
         m.put(1, "Hello");
-        s.commit();
-        long v = s.getCurrentVersion();
         m.put(2, "World.");
-        Thread.sleep(5);
-        // must not store, as nothing has been committed yet
-        s.closeImmediately();
+        s.commit();
+        s.close();
+
         s = new MVStore.Builder().
                 fileName(fileName).
                 open();
-        s.setAutoCommitDelay(1);
+        s.setAutoCommitDelay(2);
         m = s.openMap("data");
         assertEquals("World.", m.get(2));
         m.put(2, "World");
         s.commit();
-        v = s.getCurrentVersion();
+        long v = s.getCurrentVersion();
+        long time = System.currentTimeMillis();
         m.put(3, "!");
 
-        for (int i = 100; i > 0; i--) {
+        for (int i = 200; i > 0; i--) {
             if (s.getCurrentVersion() > v) {
                 break;
             }
-            if (i < 10) {
+            long diff = System.currentTimeMillis() - time;
+            if (diff > 1000) {
                 fail();
             }
-            Thread.sleep(1);
+            sleep(10);
         }
         s.closeImmediately();
 
@@ -684,7 +688,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testEncryptedFile() {
-        String fileName = getBaseDir() + "/testEncryptedFile.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s;
         MVMap<Integer, String> m;
@@ -741,7 +745,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testFileFormatChange() {
-        String fileName = getBaseDir() + "/testFileFormatChange.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s;
         MVMap<Integer, Integer> m;
@@ -753,15 +757,7 @@ public class TestMVStore extends TestBase {
         int format = Integer.parseInt(header.get("format").toString());
         assertEquals(1, format);
         header.put("format", Integer.toString(format + 1));
-        // this is to ensure the file header is overwritten
-        // the header is written at least every 20 commits
-        for (int i = 0; i < 30; i++) {
-            if (i > 5) {
-                s.setRetentionTime(0);
-            }
-            m.put(10, 100 * i);
-            s.commit();
-        }
+        forceWriteStoreHeader(s);
         s.close();
         try {
             openStore(fileName).close();
@@ -774,7 +770,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testRecreateMap() {
-        String fileName = getBaseDir() + "/testRecreateMap.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = openStore(fileName);
         MVMap<Integer, Integer> m = s.openMap("test");
@@ -805,7 +801,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testCustomMapType() {
-        String fileName = getBaseDir() + "/testMapType.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = openStore(fileName);
         SequenceMap seq = s.openMap("data", new SequenceMap.Builder());
@@ -818,7 +814,8 @@ public class TestMVStore extends TestBase {
     }
 
     private void testCacheSize() {
-        String fileName = getBaseDir() + "/testCacheSize.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         MVStore s;
         MVMap<Integer, String> map;
         s = new MVStore.Builder().
@@ -857,7 +854,8 @@ public class TestMVStore extends TestBase {
     }
 
     private void testConcurrentOpen() {
-        String fileName = getBaseDir() + "/testConcurrentOpen.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         MVStore s = new MVStore.Builder().fileName(fileName).open();
         try {
             MVStore s1 = new MVStore.Builder().fileName(fileName).open();
@@ -881,7 +879,8 @@ public class TestMVStore extends TestBase {
     }
 
     private void testFileHeader() {
-        String fileName = getBaseDir() + "/testFileHeader.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         MVStore s = openStore(fileName);
         s.setRetentionTime(Integer.MAX_VALUE);
         long time = System.currentTimeMillis();
@@ -890,17 +889,7 @@ public class TestMVStore extends TestBase {
         long creationTime = (Long) m.get("created");
         assertTrue(Math.abs(time - creationTime) < 100);
         m.put("test", "123");
-        MVMap<Integer, Integer> map = s.openMap("test");
-        map.put(10, 100);
-        // this is to ensure the file header is overwritten
-        // the header is written at least every 20 commits
-        for (int i = 0; i < 30; i++) {
-            if (i > 5) {
-                s.setRetentionTime(0);
-            }
-            map.put(10, 110);
-            s.commit();
-        }
+        forceWriteStoreHeader(s);
         s.close();
         s = openStore(fileName);
         Object test = s.getStoreHeader().get("test");
@@ -909,8 +898,40 @@ public class TestMVStore extends TestBase {
         s.close();
     }
 
+    private static void forceWriteStoreHeader(MVStore s) {
+        MVMap<Integer, Integer> map = s.openMap("dummy");
+        map.put(10, 100);
+        // this is to ensure the file header is overwritten
+        // the header is written at least every 20 commits
+        for (int i = 0; i < 30; i++) {
+            if (i > 5) {
+                s.setRetentionTime(0);
+                // ensure that the next save time is different,
+                // so that blocks can be reclaimed
+                // (on Windows, resolution is 10 ms)
+                sleep(1);
+            }
+            map.put(10, 110);
+            s.commit();
+        }
+        s.removeMap(map);
+        s.commit();
+    }
+
+    private static void sleep(long ms) {
+        // on Windows, need to sleep in some cases,
+        // mainly because the milliseconds resolution of
+        // System.currentTimeMillis is 10 ms.
+        try {
+            Thread.sleep(ms);
+        } catch (InterruptedException e) {
+            // ignore
+        }
+    }
+
     private void testFileHeaderCorruption() throws Exception {
-        String fileName = getBaseDir() + "/testFileHeader.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         MVStore s = new MVStore.Builder().
                 fileName(fileName).pageSplitSize(1000).autoCommitDisabled().open();
         s.setRetentionTime(0);
@@ -924,7 +945,7 @@ public class TestMVStore extends TestBase {
         }
         FileStore fs = s.getFileStore();
         long size = fs.getFile().size();
-        for (int i = 0; i < 10; i++) {
+        for (int i = 0; i < 100; i++) {
             map = s.openMap("test" + i);
             s.removeMap(map);
             s.commit();
@@ -933,6 +954,10 @@ public class TestMVStore extends TestBase {
                 break;
             }
         }
+        // the last chunk is at the end
+        s.setReuseSpace(false);
+        map = s.openMap("test2");
+        map.put(1, new byte[1000]);
         s.close();
         FilePath f = FilePath.get(fileName);
         int blockSize = 4 * 1024;
@@ -961,6 +986,8 @@ public class TestMVStore extends TestBase {
                 s = openStore(fileName);
                 map = s.openMap("test");
                 assertEquals(100, map.get(0).length);
+                map = s.openMap("test2");
+                assertFalse(map.containsKey(1));
                 s.close();
             } else {
                 // both headers are corrupt
@@ -1100,7 +1127,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testStoreVersion() {
-        String fileName = getBaseDir() + "/testStoreVersion.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = MVStore.open(fileName);
         assertEquals(0, s.getCurrentVersion());
@@ -1144,7 +1171,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testObjects() {
-        String fileName = getBaseDir() + "/testObjects.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s;
         Map<Object, Object> map;
@@ -1167,7 +1194,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testExample() {
-        String fileName = getBaseDir() + "/testExample.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
 
         // open the store (in-memory if fileName is null)
@@ -1190,7 +1217,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testExampleMvcc() {
-        String fileName = getBaseDir() + "/testExampleMvcc.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
 
         // open the store (in-memory if fileName is null)
@@ -1236,7 +1263,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testOpenStoreCloseLoop() {
-        String fileName = getBaseDir() + "/testOpenClose.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         for (int k = 0; k < 1; k++) {
             // long t = System.currentTimeMillis();
@@ -1282,7 +1309,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testVersion() {
-        String fileName = getBaseDir() + "/testVersion.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s;
         s = openStore(fileName);
@@ -1347,7 +1374,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testTruncateFile() {
-        String fileName = getBaseDir() + "/testTruncate.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s;
         MVMap<Integer, String> m;
@@ -1366,10 +1393,11 @@ public class TestMVStore extends TestBase {
         m = s.openMap("data");
         for (int i = 0; i < 10; i++) {
             if (i % 4 != 0) {
+                sleep(2);
                 m.remove(i);
+                s.commit();
             }
         }
-        s.commit();
         assertTrue(s.compact(100, 50 * 1024));
         s.close();
         long len2 = FileUtils.size(fileName);
@@ -1377,7 +1405,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testFastDelete() {
-        String fileName = getBaseDir() + "/testFastDelete.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s;
         MVMap<Integer, String> m;
@@ -1399,7 +1427,7 @@ public class TestMVStore extends TestBase {
         assertEquals(0, m.size());
         s.commit();
         // ensure only nodes are read, but not leaves
-        assertEquals(41, s.getFileStore().getReadCount());
+        assertEquals(45, s.getFileStore().getReadCount());
         assertTrue(s.getFileStore().getWriteCount() < 5);
         s.close();
     }
@@ -1419,7 +1447,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testRollbackStored() {
-        String fileName = getBaseDir() + "/testRollback.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVMap<String, String> meta;
         MVStore s = openStore(fileName);
@@ -1512,7 +1540,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testRollbackInMemory() {
-        String fileName = getBaseDir() + "/testRollback.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = openStore(fileName, 5);
         s.setAutoCommitDelay(0);
@@ -1553,7 +1581,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testMeta() {
-        String fileName = getBaseDir() + "/testMeta.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = openStore(fileName);
         s.setRetentionTime(Integer.MAX_VALUE);
@@ -1564,7 +1592,6 @@ public class TestMVStore extends TestBase {
         data.put("2", "World");
         s.commit();
         assertEquals(1, s.getCurrentVersion());
-        assertFalse(m.containsKey("chunk.2"));
 
         assertEquals("[data]", s.getMapNames().toString());
         assertEquals("data", s.getMapName(data.getId()));
@@ -1584,8 +1611,6 @@ public class TestMVStore extends TestBase {
         s.rollbackTo(1);
         assertEquals("Hello", data.get("1"));
         assertEquals("World", data.get("2"));
-        assertFalse(m.containsKey("chunk.1"));
-        assertFalse(m.containsKey("chunk.2"));
 
         s.close();
     }
@@ -1623,7 +1648,8 @@ public class TestMVStore extends TestBase {
     }
 
     private void testLargeImport() {
-        String fileName = getBaseDir() + "/testImport.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
+        FileUtils.delete(fileName);
         int len = 1000;
         for (int j = 0; j < 5; j++) {
             FileUtils.delete(fileName);
@@ -1659,7 +1685,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testBtreeStore() {
-        String fileName = getBaseDir() + "/testBtreeStore.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = openStore(fileName);
         s.close();
@@ -1707,7 +1733,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testCompactMapNotOpen() {
-        String fileName = getBaseDir() + "/testCompactNotOpen.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = openStore(fileName, 1000);
         MVMap<Integer, String> m = s.openMap("data");
@@ -1743,6 +1769,7 @@ public class TestMVStore extends TestBase {
 
         m = s.openMap("data");
         for (int i = 0; i < 10; i++) {
+            sleep(1);
             boolean result = s.compact(50, 50 * 1024);
             if (!result) {
                 break;
@@ -1767,10 +1794,11 @@ public class TestMVStore extends TestBase {
     }
 
     private void testCompact() {
-        String fileName = getBaseDir() + "/testCompact.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         long initialLength = 0;
         for (int j = 0; j < 20; j++) {
+            sleep(2);
             MVStore s = openStore(fileName);
             s.setRetentionTime(0);
             MVMap<Integer, String> m = s.openMap("data");
@@ -1808,10 +1836,11 @@ public class TestMVStore extends TestBase {
     }
 
     private void testReuseSpace() {
-        String fileName = getBaseDir() + "/testReuseSpace.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         long initialLength = 0;
         for (int j = 0; j < 20; j++) {
+            sleep(2);
             MVStore s = openStore(fileName);
             s.setRetentionTime(0);
             MVMap<Integer, String> m = s.openMap("data");
@@ -1829,13 +1858,13 @@ public class TestMVStore extends TestBase {
                 initialLength = len;
             } else {
                 assertTrue("len: " + len + " initial: " + initialLength + " j: " + j,
-                        len <= initialLength * 2);
+                        len <= initialLength * 5);
             }
         }
     }
 
     private void testRandom() {
-        String fileName = getBaseDir() + "/testRandom.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = openStore(fileName);
         MVMap<Integer, Integer> m = s.openMap("data");
@@ -1898,7 +1927,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testKeyValueClasses() {
-        String fileName = getBaseDir() + "/testKeyValueClasses.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = openStore(fileName);
         MVMap<Integer, String> is = s.openMap("intString");
@@ -1923,7 +1952,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testIterate() {
-        String fileName = getBaseDir() + "/testIterate.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = openStore(fileName);
         MVMap<Integer, String> m = s.openMap("data");
@@ -1956,7 +1985,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testCloseTwice() {
-        String fileName = getBaseDir() + "/testCloseTwice.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = openStore(fileName);
         MVMap<Integer, String> m = s.openMap("data");
@@ -1969,7 +1998,7 @@ public class TestMVStore extends TestBase {
     }
 
     private void testSimple() {
-        String fileName = getBaseDir() + "/testSimple.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore s = openStore(fileName);
         MVMap<Integer, String> m = s.openMap("data");
@@ -1998,30 +2027,34 @@ public class TestMVStore extends TestBase {
         if (!config.big) {
             return;
         }
-        String fileName = getBaseDir() + "/testLargerThan2G.h3";
+        String fileName = getBaseDir() + "/" + getTestName();
         FileUtils.delete(fileName);
         MVStore store = new MVStore.Builder().cacheSize(16).
                 fileName(fileName).open();
-        MVMap<Integer, String> map = store.openMap("test");
-        long last = System.currentTimeMillis();
-        String data = new String(new char[2500]).replace((char) 0, 'x');
-        for (int i = 0;; i++) {
-            map.put(i, data);
-            if (i % 10000 == 0) {
-                store.commit();
-                long time = System.currentTimeMillis();
-                if (time - last > 2000) {
-                    long mb = store.getFileStore().size() / 1024 / 1024;
-                    trace(mb + "/4500");
-                    if (mb > 4500) {
-                        break;
+        try {
+            MVMap<Integer, String> map = store.openMap("test");
+            long last = System.currentTimeMillis();
+            String data = new String(new char[2500]).replace((char) 0, 'x');
+            for (int i = 0;; i++) {
+                map.put(i, data);
+                if (i % 10000 == 0) {
+                    store.commit();
+                    long time = System.currentTimeMillis();
+                    if (time - last > 2000) {
+                        long mb = store.getFileStore().size() / 1024 / 1024;
+                        trace(mb + "/4500");
+                        if (mb > 4500) {
+                            break;
+                        }
+                        last = time;
                     }
-                    last = time;
                 }
             }
+            store.commit();
+            store.close();
+        } finally {
+            store.closeImmediately();
         }
-        store.commit();
-        store.close();
         FileUtils.delete(fileName);
     }
 
