@@ -45,6 +45,8 @@ public class TestTransactionStore extends TestBase {
     @Override
     public void test() throws Exception {
         FileUtils.createDirectories(getBaseDir());
+        testConcurrentAddRemove();
+        testConcurrentAdd();
         testCountWithOpenTransactions();
         testConcurrentUpdate();
         testRepeatedChange();
@@ -58,6 +60,110 @@ public class TestTransactionStore extends TestBase {
         testConcurrentTransactionsReadCommitted();
         testSingleConnection();
         testCompareWithPostgreSQL();
+        testStoreMultiThreadedReads();
+    }
+
+    private void testConcurrentAddRemove() throws InterruptedException {
+        MVStore s = MVStore.open(null);
+        int threadCount = 3;
+        final int keyCount = 2;
+        final TransactionStore ts = new TransactionStore(s);
+        ts.init();
+
+        final Random r = new Random(1);
+
+        Task[] tasks = new Task[threadCount];
+        for (int i = 0; i < threadCount; i++) {
+            Task task = new Task() {
+
+                @Override
+                public void call() throws Exception {
+                    TransactionMap<Integer, Integer> map = null;
+                    while (!stop) {
+                        Transaction tx = ts.begin();
+                        map = tx.openMap("data");
+                        int k = r.nextInt(keyCount);
+                        try {
+                            map.remove(k);
+                            map.put(k, r.nextInt());
+                        } catch (IllegalStateException e) {
+                            // ignore and retry
+                        }
+                        tx.commit();
+                    }
+                }
+
+            };
+            task.execute();
+            tasks[i] = task;
+        }
+        Thread.sleep(1000);
+        for (Task t : tasks) {
+            t.get();
+        }
+        s.close();
+    }
+
+    private void testConcurrentAdd() {
+        MVStore s;
+        s = MVStore.open(null);
+        final TransactionStore ts = new TransactionStore(s);
+        ts.init();
+
+        final Random r = new Random(1);
+
+        final AtomicInteger key = new AtomicInteger();
+        final AtomicInteger failCount = new AtomicInteger();
+
+        Task task = new Task() {
+
+            @Override
+            public void call() throws Exception {
+                Transaction tx = null;
+                TransactionMap<Integer, Integer> map = null;
+                while (!stop) {
+                    int k = key.get();
+                    tx = ts.begin();
+                    map = tx.openMap("data");
+                    try {
+                        map.put(k, r.nextInt());
+                    } catch (IllegalStateException e) {
+                        failCount.incrementAndGet();
+                        // ignore and retry
+                    }
+                    tx.commit();
+                }
+            }
+
+        };
+        task.execute();
+        Transaction tx = null;
+        int count = 100000;
+        TransactionMap<Integer, Integer> map = null;
+        for (int i = 0; i < count; i++) {
+            int k = i;
+            key.set(k);
+            tx = ts.begin();
+            map = tx.openMap("data");
+            try {
+                map.put(k, r.nextInt());
+            } catch (IllegalStateException e) {
+                failCount.incrementAndGet();
+                // ignore and retry
+            }
+            tx.commit();
+            if (failCount.get() > 0 && i > 4000) {
+                // stop earlier, if possible
+                count = i;
+                break;
+            }
+        }
+        // we expect at least 10% the operations were successful
+        assertTrue(failCount.toString() + " >= " + (count * 0.9),
+                failCount.get() < count * 0.9);
+        // we expect at least a few failures
+        assertTrue(failCount.toString(), failCount.get() > 0);
+        s.close();
     }
 
     private void testCountWithOpenTransactions() {
@@ -154,15 +260,21 @@ public class TestTransactionStore extends TestBase {
         ts = new TransactionStore(s);
         ts.init();
         ts.setMaxTransactionId(16);
+        ArrayList<Transaction> openList = new ArrayList<Transaction>();
         for (int i = 0, j = 1; i < 64; i++) {
             Transaction t = ts.begin();
+            openList.add(t);
             assertEquals(j, t.getId());
-            t.commit();
             j++;
             if (j > 16) {
                 j = 1;
             }
+            if (openList.size() >= 16) {
+                t = openList.remove(0);
+                t.commit();
+            }
         }
+
         s = MVStore.open(null);
         ts = new TransactionStore(s);
         ts.init();
@@ -170,10 +282,10 @@ public class TestTransactionStore extends TestBase {
         ArrayList<Transaction> fifo = New.arrayList();
         int open = 0;
         for (int i = 0; i < 64; i++) {
-            Transaction t = ts.begin();
+            Transaction t = null;
             if (open >= 16) {
                 try {
-                    t.openMap("data").put(i, i);
+                    t = ts.begin();
                     fail();
                 } catch (IllegalStateException e) {
                     // expected - too many open
@@ -182,9 +294,10 @@ public class TestTransactionStore extends TestBase {
                 first.commit();
                 open--;
             }
+            t = ts.begin();
+            t.openMap("data").put(i, i);
             fifo.add(t);
             open++;
-            t.openMap("data").put(i, i);
         }
         s.close();
     }
@@ -514,6 +627,8 @@ public class TestTransactionStore extends TestBase {
         assertEquals("first transaction", txOld.getName());
         txOld.prepare();
         assertEquals(Transaction.STATUS_PREPARED, txOld.getStatus());
+        txOld = list.get(1);
+        txOld.commit();
         s.commit();
         s.close();
 
@@ -522,6 +637,7 @@ public class TestTransactionStore extends TestBase {
         ts.init();
         tx = ts.begin();
         m = tx.openMap("test");
+        m.put("3", "Test");
         assertEquals(2, tx.getId());
         list = ts.getOpenTransactions();
         assertEquals(2, list.size());
@@ -872,4 +988,58 @@ public class TestTransactionStore extends TestBase {
         s.close();
     }
 
+    private void testStoreMultiThreadedReads() throws Exception {
+        MVStore s = MVStore.open(null);
+        final TransactionStore ts = new TransactionStore(s);
+
+        ts.init();
+        Transaction t = ts.begin();
+        TransactionMap<Integer, Integer> mapA = t.openMap("a");
+        mapA.put(1, 0);
+        t.commit();
+
+        Task task = new Task() {
+            @Override
+            public void call() throws Exception {
+                for (int i = 0; !stop; i++) {
+                    Transaction tx = ts.begin();
+                    TransactionMap<Integer, Integer> mapA = tx.openMap("a");
+                    while (!mapA.tryPut(1, i)) {
+                        // repeat
+                    }
+                    tx.commit();
+
+                    // map B transaction
+                    // the other thread will get a map A uncommitted value,
+                    // but by the time it tries to walk back to the committed
+                    // value, the undoLog has changed
+                    tx = ts.begin();
+                    TransactionMap<Integer, Integer> mapB = tx.openMap("b");
+                    // put a new value to the map; this will cause a map B
+                    // undoLog entry to be created with a null pre-image value
+                    mapB.tryPut(i, -i);
+                    // this is where the real race condition occurs:
+                    // some other thread might get the B log entry
+                    // for this transaction rather than the uncommitted A log
+                    // entry it is expecting
+                    tx.commit();
+                }
+            }
+        };
+        task.execute();
+        try {
+            for (int i = 0; i < 10000; i++) {
+                Transaction tx = ts.begin();
+                mapA = tx.openMap("a");
+                if (mapA.get(1) == null) {
+                    throw new AssertionError("key not found");
+                }
+                tx.commit();
+            }
+        } finally {
+            task.get();
+        }
+        ts.close();
+    }
+
 }
